SystemOrganization addCategory: #'Mendel-Tests'!

TestCase subclass: #MendelClassLevelTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelClassLevelTest methodsFor: 'testing family traits' stamp: 'AndyKellens 6/21/2012 17:35'!
testClassDominantTraits
	| dominantTraits |
	dominantTraits := (Genotype forEntity: MendelSubclassB) dominantTraits.
	
	self assert: (dominantTraits size = 6).
	self assert: (dominantTraits anySatisfy: [:trait | trait isKeywordTrait and:[trait value = 'Subclass']]).
	self assert: (dominantTraits anySatisfy: [:trait | trait isHierarchyTrait and:[trait value = MendelTestHierarchyRoot]]).
	self assert: (dominantTraits anySatisfy: [:trait | trait isImplementsTrait and:[trait value = #foo]]).
	self assert: (dominantTraits anySatisfy: [:trait | trait isKeywordTrait and:[trait value = 'Mendel']]).
	self assert: (dominantTraits anySatisfy: [:trait | trait isKeywordTrait and:[trait value = 'Mendel']]).
	self assert: (dominantTraits anySatisfy: [:trait | trait isUnderstandsTrait and:[trait value = #foo]]).
	self assert: (dominantTraits anySatisfy: [:trait | trait isUnderstandsTrait and:[trait value = #bar]]).! !

!MendelClassLevelTest methodsFor: 'testing family traits' stamp: 'AndyKellens 6/20/2012 14:24'!
testClassDominantTraits2
	| dominantTraits |
	dominantTraits := (Genotype forEntity: MendelSubclassD) dominantTraits.
	
	self assert: (dominantTraits size = 3).
	self assert: (dominantTraits anySatisfy: [:trait | trait isKeywordTrait and:[trait value = 'Subclass']]).
	self assert: (dominantTraits anySatisfy: [:trait | trait isHierarchyTrait and:[trait value = MendelTestHierarchyRoot]]).
	self assert: (dominantTraits anySatisfy: [:trait | trait isKeywordTrait and:[trait value = 'Mendel']]).! !

!MendelClassLevelTest methodsFor: 'testing family traits' stamp: 'AndyKellens 6/20/2012 14:54'!
testClassFamilyTraits
	| familyTraits |
	familyTraits := (Genotype forEntity: MendelSubclassB) familyTraits.
	"There should be at most four classes for each of the traits"
	self assert:(familyTraits values allSatisfy:[:classes | classes size <= 5]).
	
	"There are 13 properties"
	self assert: (familyTraits size = 13).
	
	"Four classes in the family implement foo"
	self assert:(familyTraits at: (ImplementsMessageTrait value:#foo)) size = 4.
	
	"All classes in the family are called Mendel*"
	self assert:(familyTraits at: (KeywordTrait value:'Mendel')) size = 5.
	
	"All classes in the family are in the hierarchy of MendelTestHierarchyRoot"
	self assert:(familyTraits at: (InHierarchyTrait value:MendelTestHierarchyRoot)) size = 5
	! !

!MendelClassLevelTest methodsFor: 'testing family traits' stamp: 'AndyKellens 6/20/2012 14:55'!
testClassFamilyTraits2
	| familyTraits |
	familyTraits := (Genotype forEntity: MendelSubclassD) familyTraits.
	"There should be at most seven classes for each of the traits"
	self assert:(familyTraits values allSatisfy:[:classes | classes size <= 7]).

	"There are 15 properties"
	self assert: (familyTraits size = 15).
	
	"Four classes in the family implement foo"
	self assert:(familyTraits at: (ImplementsMessageTrait value:#foo)) size = 5.
	
	"All classes in the family are called Mendel*"
	self assert:(familyTraits at: (KeywordTrait value:'Mendel')) size = 7.
	
	"All classes in the family are in the hierarchy of MendelTestHierarchyRoot"
	self assert:(familyTraits at: (InHierarchyTrait value:MendelTestHierarchyRoot)) size = 7
	! !

!MendelClassLevelTest methodsFor: 'testing family traits' stamp: 'AndyKellens 6/21/2012 17:36'!
testClassRecessiveTraits
	| recessiveTraits |
	recessiveTraits := (Genotype forEntity: MendelSubclassB) recessiveTraits.
	
	self assert: (recessiveTraits size = 1).
	self assert: (recessiveTraits anySatisfy: [:trait | trait isImplementsTrait and:[trait value = #bar]])! !

!MendelClassLevelTest methodsFor: 'testing recommendations' stamp: 'AndyKellens 6/21/2012 17:35'!
testClassRecommendations
	| recommendations |
	recommendations := MendelSubclassB mendelRecommendations.
	self assert: (recommendations entity = MendelSubclassB).
	self assert: (recommendations entityTraits size = 4).
	
	self assert: (recommendations missing size = 3).
	self assert: (recommendations missing anySatisfy: [:recommendation | recommendation = (ImplementsMessageTrait value:#foo)]).
	self assert: (recommendations missing anySatisfy: [:recommendation | recommendation = (UnderstandsMessageTrait value:#foo)]).
	self assert: (recommendations missing anySatisfy: [:recommendation | recommendation = (UnderstandsMessageTrait value:#bar)]).
	
	self assert: (recommendations suggested size = 1).
	self assert: (recommendations suggested anySatisfy: [:recommendation | recommendation = (ImplementsMessageTrait value:#bar)]).
	
	! !

!MendelClassLevelTest methodsFor: 'testing family' stamp: 'AndyKellens 6/20/2012 14:53'!
testFamilyOfAClassWithoutMethods
	"The family of MendelSubclassB should be 5 classes"
	| thefamily |
	thefamily := MendelSubclassB mendelFamily.
	self assert: (thefamily size = 5).
	self assert: (thefamily includes: MendelSubclassA1).
	self assert: (thefamily includes: MendelSubclassA2).
	self assert: (thefamily includes: MendelSubclassD).
	self assert: (thefamily includes: MendelSubclassD1).
	self assert: (thefamily includes: MendelSubclassE)! !

!MendelClassLevelTest methodsFor: 'testing family' stamp: 'AndyKellens 6/20/2012 14:53'!
testFamilyOfARegularClass
	"The family of MendelSubclassD should be 7 classes"
	| thefamily |
	thefamily := MendelSubclassD mendelFamily.
	self assert: (thefamily size = 7).
	self assert: (thefamily includes: MendelSubclassB).
	self assert: (thefamily includes: MendelSubclassB1).
	self assert: (thefamily includes: MendelSubclassB2).
	self assert: (thefamily includes: MendelSubclassB3).
	self assert: (thefamily includes: MendelSubclassA1).
	self assert: (thefamily includes: MendelSubclassA2).
	self assert: (thefamily includes: MendelSubclassE)! !

!MendelClassLevelTest methodsFor: 'testing family' stamp: 'AndyKellens 6/19/2012 17:11'!
testFamilyOfAnAbstractClass
	"The family of MendelSubclassC should be 1 class"
	| thefamily |
	thefamily := MendelSubclassC mendelFamily.
	self assert: (thefamily size = 1).
	self assert: (thefamily includes: MendelSubclassA)! !

TestCase subclass: #MendelMethodLevelTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelMethodLevelTest methodsFor: 'testing family' stamp: 'AndyKellens 6/19/2012 17:46'!
testFamilyOfAbstractMethod
	"The family of the method  MendelSubclassC>>foo should consists of 1 method"
	| thefamily |
	thefamily := (MendelSubclassC >> #foo) mendelFamily.
	self assert: (thefamily size = 1).
	self assert: (thefamily includes: (MendelSubclassA >> #foo))! !

!MendelMethodLevelTest methodsFor: 'testing family' stamp: 'AndyKellens 6/20/2012 15:00'!
testFamilyOfRegularMethod
	"The family of the method  MendelSubclassD>>foo should consists of 4 methods"
	| thefamily |
	thefamily := (MendelSubclassD >> #foo) mendelFamily.
	self assert: (thefamily size = 5).
	self assert: (thefamily includes: (MendelSubclassA1 >> #foo)).
	self assert: (thefamily includes: (MendelSubclassA2 >> #foo)).
	self assert: (thefamily includes: (MendelSubclassB1 >> #foo)).
	self assert: (thefamily includes: (MendelSubclassB2 >> #foo)).
	self assert: (thefamily includes: (MendelSubclassE >> #foo)).
	! !

!MendelMethodLevelTest methodsFor: 'testing family traits' stamp: 'AndyKellens 6/20/2012 15:06'!
testMethodDominantTraits
	| dominantTraits |
	dominantTraits := (Genotype forEntity: MendelSubclassD >> #foo) dominantTraits.
	
	self assert: (dominantTraits size = 3).
	self assert: (dominantTraits anySatisfy: [:trait | trait isImplementsTrait and:[trait value = #foo]]).
	self assert: (dominantTraits anySatisfy: [:trait | trait isCallsTrait  and:[trait value = #doSomething]]).
	self assert: (dominantTraits anySatisfy: [:trait | trait isProtocolTrait  and:[trait value = #'my methods']]).! !

!MendelMethodLevelTest methodsFor: 'testing family traits' stamp: 'AndyKellens 6/20/2012 15:08'!
testMethodFamilyTraits
	| familyTraits |
	familyTraits := (Genotype forEntity: MendelSubclassD >> #foo) familyTraits.
	"There should be at most four methods for each of the traits"
	self assert:(familyTraits values allSatisfy:[:methods | methods size <= 5]).

	"There are 3 properties"
	self assert: (familyTraits size = 4).
	
	"Four methods in the family implement foo"
	self assert:(familyTraits at: (ImplementsMessageTrait value:#foo)) size = 5.
	
	"All methods in the family are in protocol my methods"
	self assert:(familyTraits at: (InProtocolTrait value:#'my methods')) size = 5.
	
	"All methods in the family call doSomething"
	self assert:(familyTraits at: (CallsTrait value:#doSomething)) size = 5.
	
	"Three methods in the family call doSomethingElse"
	self assert:(familyTraits at: (CallsTrait value:#doSomethingElse)) size = 3
	! !

!MendelMethodLevelTest methodsFor: 'testing family traits' stamp: 'AndyKellens 6/20/2012 15:08'!
testMethodRecessiveTraits
	| recessiveTraits |
	recessiveTraits := (Genotype forEntity: MendelSubclassD >> #foo) recessiveTraits.
	self assert: (recessiveTraits size = 1).
	self assert: (recessiveTraits anySatisfy: [:trait | trait isCallsTrait  and:[trait value = #doSomethingElse]])! !

!MendelMethodLevelTest methodsFor: 'testing recommendations' stamp: 'AndyKellens 6/21/2012 17:41'!
testMethodRecommendations
	| recommendations |
	recommendations := (MendelSubclassE >> #foo) mendelRecommendations.
	self assert: (recommendations entity = (MendelSubclassE >> #foo)).
	self assert: (recommendations entityTraits size = 3).
	self assert: (recommendations suggested isEmpty).
	self assert: (recommendations missing size = 1).
	self assert: (recommendations missing anySatisfy: [:recommendation | recommendation = (CallsTrait value: #doSomethingElse)])! !

TestCase subclass: #MendelTraitTestCase
	instanceVariableNames: 'entityTraits'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

MendelTraitTestCase subclass: #MendelClassMethodTraitsTest
	instanceVariableNames: 'entityTraits2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelClassMethodTraitsTest methodsFor: 'private' stamp: 'AndyKellens 6/19/2012 16:07'!
createEntityTraits
	^(MendelTestSubClass >> #bar) mendelTraits ! !

!MendelClassMethodTraitsTest methodsFor: 'private' stamp: 'AndyKellens 6/19/2012 16:07'!
createEntityTraits2
	^(MendelTestSubClass >> #fooBlaBlar) mendelTraits ! !

!MendelClassMethodTraitsTest methodsFor: 'running' stamp: 'AndyKellens 6/19/2012 16:07'!
setUp
	super setUp.
	entityTraits2 := self createEntityTraits2! !

!MendelClassMethodTraitsTest methodsFor: 'testing traits' stamp: 'AndyKellens 6/19/2012 16:09'!
testCalls
	| calls |
	calls := entityTraits select:[:trait | trait isCallsTrait].
	self assert: (calls size = 4).
	self assert: (calls anySatisfy: [:call | call value = #do]).
	self assert: (calls anySatisfy: [:call | call value = #something]).
	self assert: (calls anySatisfy: [:call | call value = #then]).
	self assert: (calls anySatisfy: [:call | call value = #else]).
	
! !

!MendelClassMethodTraitsTest methodsFor: 'testing traits' stamp: 'AndyKellens 6/19/2012 16:10'!
testImplements
	| implements |
	implements := entityTraits select:[:trait | trait isImplementsTrait].
	self assert: (implements size = 1).
	self assert: (implements anySatisfy: [:impl | impl value = #bar]).
	! !

!MendelClassMethodTraitsTest methodsFor: 'testing traits' stamp: 'AndyKellens 6/19/2012 16:12'!
testKeywords
	| keywords |
	keywords := entityTraits2 select:[:trait | trait isKeywordTrait].
	self assert: (keywords size = 1).
	self assert: (keywords anySatisfy: [:kw | kw value = 'Blar']).
	! !

!MendelClassMethodTraitsTest methodsFor: 'testing traits' stamp: 'AndyKellens 6/19/2012 16:11'!
testProtocol
	| protocols |
	protocols := entityTraits select:[:trait | trait isProtocolTrait].
	self assert: (protocols size = 1).
	self assert: (protocols anySatisfy: [:impl | impl value = #'protocol of foo']).
	! !

!MendelClassMethodTraitsTest methodsFor: 'testing traits' stamp: 'AndyKellens 6/19/2012 16:21'!
testRefers
	| refers |
	refers := entityTraits2 select:[:trait | trait isRefersClassTrait].
	self assert: (refers size = 1).
	self assert: (refers anySatisfy: [:ref | ref value = OrderedCollection]).
	! !

MendelTraitTestCase subclass: #MendelClassTraitsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelClassTraitsTest methodsFor: 'private' stamp: 'AndyKellens 6/18/2012 13:02'!
createEntityTraits
	^MendelTestSubClass mendelTraits! !

!MendelClassTraitsTest methodsFor: 'testing traits' stamp: 'AndyKellens 6/18/2012 13:40'!
testHierarchy
	| hierarchy |
	hierarchy := entityTraits select:[:trait | trait isHierarchyTrait].
	self assert: (hierarchy size = 2).
	self assert: (hierarchy anySatisfy: [:trait | trait value = MendelTestClass]).
	self assert: (hierarchy anySatisfy: [:trait | trait value = MendelTestSubClass])
	! !

!MendelClassTraitsTest methodsFor: 'testing traits' stamp: 'AndyKellens 6/19/2012 16:06'!
testImplementsMessages
	| implements |
	implements := entityTraits select:[:trait | trait isImplementsTrait].
	self assert: (implements size = 2).
	self assert: (implements anySatisfy: [:trait | trait value = #fooBlaBlar ]).
	self assert: (implements anySatisfy: [:trait | trait value = #bar ]).! !

!MendelClassTraitsTest methodsFor: 'testing traits' stamp: 'AndyKellens 6/18/2012 13:37'!
testKeywords
	| keywords |
	keywords := entityTraits select:[:trait | trait isKeywordTrait].
	self assert: (keywords size = 3).
	self assert: (keywords anySatisfy: [:trait | trait value = 'Mendel' ]).
	self assert: (keywords anySatisfy: [:trait | trait value = 'Test' ]).
	self assert: (keywords anySatisfy: [:trait | trait value = 'Class' ]).
	self deny: (keywords anySatisfy: [:trait | trait value = 'Sub' ]).! !

!MendelClassTraitsTest methodsFor: 'testing traits' stamp: 'AndyKellens 6/21/2012 17:39'!
testUnderstandsMessages
	| understands |
	understands := entityTraits select:[:trait | trait isUnderstandsTrait].
	self assert: (understands size = 3).
	self assert: (understands anySatisfy: [:trait | trait value = #alsoUnderstandsThisMessage ]).
	self assert: (understands anySatisfy: [:trait | trait value = #bar ]).
	self assert: (understands anySatisfy: [:trait | trait value = #fooBlaBlar ]).! !

!MendelTraitTestCase methodsFor: 'private' stamp: 'AndyKellens 6/18/2012 12:55'!
createEntityTraits
	self subclassResponsibility ! !

!MendelTraitTestCase methodsFor: 'running' stamp: 'AndyKellens 6/18/2012 12:55'!
setUp
	entityTraits := self createEntityTraits! !

Object subclass: #MendelTestClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelTestClass methodsFor: 'bla' stamp: 'AndyKellens 6/18/2012 12:54'!
alsoUnderstandsThisMessage! !

MendelTestClass subclass: #MendelTestSubClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelTestSubClass methodsFor: 'protocol of foo' stamp: 'AndyKellens 6/18/2012 12:47'!
bar
 	self do;something;then;do;something;else! !

!MendelTestSubClass methodsFor: 'protocol of foo' stamp: 'AndyKellens 6/19/2012 16:06'!
fooBlaBlar
	^OrderedCollection new! !

Object subclass: #MendelTestHierarchyRoot
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

MendelTestHierarchyRoot subclass: #MendelSubclassA
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelSubclassA methodsFor: 'my methods' stamp: 'AndyKellens 6/20/2012 14:19'!
bar! !

!MendelSubclassA methodsFor: 'my methods' stamp: 'AndyKellens 6/19/2012 16:45'!
foo
	self subclassResponsibility ! !

MendelSubclassA subclass: #MendelSubclassA1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelSubclassA1 methodsFor: 'my methods' stamp: 'AndyKellens 6/20/2012 14:19'!
bar! !

!MendelSubclassA1 methodsFor: 'my methods' stamp: 'AndyKellens 6/20/2012 14:35'!
foo 
	self doSomething;doSomethingElse! !

MendelSubclassA1 subclass: #MendelSubclassA1sub
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelSubclassA1sub methodsFor: 'my methods' stamp: 'AndyKellens 6/20/2012 14:35'!
foo 
	self doSomething;doSomethingElse! !

MendelSubclassA subclass: #MendelSubclassA2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelSubclassA2 methodsFor: 'my methods' stamp: 'AndyKellens 6/20/2012 14:19'!
bar! !

!MendelSubclassA2 methodsFor: 'my methods' stamp: 'AndyKellens 6/20/2012 15:07'!
foo 
	self doSomething; doSomethingElse! !

MendelTestHierarchyRoot subclass: #MendelSubclassB
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

MendelSubclassB subclass: #MendelSubclassB1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelSubclassB1 methodsFor: 'my methods' stamp: 'AndyKellens 6/20/2012 14:40'!
bar! !

!MendelSubclassB1 methodsFor: 'my methods' stamp: 'AndyKellens 6/20/2012 14:36'!
foo 
	self doSomething;doSomethingElse! !

MendelSubclassB subclass: #MendelSubclassB2
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelSubclassB2 methodsFor: 'my methods' stamp: 'AndyKellens 6/19/2012 16:46'!
foo 
	self doSomething! !

MendelSubclassB subclass: #MendelSubclassB3
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

MendelTestHierarchyRoot subclass: #MendelSubclassC
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelSubclassC methodsFor: 'my methods' stamp: 'AndyKellens 6/19/2012 16:45'!
foo
	self subclassResponsibility ! !

MendelTestHierarchyRoot subclass: #MendelSubclassD
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelSubclassD methodsFor: 'my methods' stamp: 'AndyKellens 6/20/2012 14:57'!
bar! !

!MendelSubclassD methodsFor: 'my methods' stamp: 'AndyKellens 6/19/2012 16:46'!
foo 
	self doSomething; doSomethingElse! !

MendelSubclassD subclass: #MendelSubclassD1
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

MendelTestHierarchyRoot subclass: #MendelSubclassE
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Mendel-Tests'!

!MendelSubclassE methodsFor: 'my methods' stamp: 'AndyKellens 6/20/2012 14:52'!
foo 
	self doSomething
! !
